%% Initialise parameters
% Classes: 
%
% simulator: updates position and velocity in NED
% range: generates range measurements (algebraic transformation) 
% ekf: Extended Kalman Filter estimating position, velocity and bias
%
% Date for latest update: 06.02.2017
% Programmed by: Katrine Seel
% Master thesis, Spring 2017 

clear all; 

%% Load measurement data 

imu = importdata('imuMatrix.mat');
qualisys = importdata('qualisysMatrix.mat');
data = importdata('dataMatrix.mat'); 

[~, N] = size(imu); 
h = 1/data(1,1); 

% Allocation of matrices
q = zeros(4,N);
pos_n = zeros(3,N);
vel_n = zeros(3,N);
acc_n = zeros(3,N);
vel_b  = zeros(3,N);
att_n = zeros(3,N);
pos_est = zeros(3,N);
vel_est = zeros(3,N);
bias_est = zeros(1,N);
pos_gen = zeros(4,N);
p = zeros(6,N);
pos_est2 = zeros(3,N);
vel_est2 = zeros(3,N); 
bias_est2 = zeros(1,N);

% Define initial conditions
vel_b0 = [0 0 0]'; 
acc_n0 = [0 0 0]';
vel_n0 = [-0.3307; 0.1078; 0.0136]; 
q0     = [1;0;0;0];         % equals 0 roll, pitch, and yaw 
pos_n0 = [-1.6581; 1.1946;-1.4999];
bias_20 = [0;0;0];
q(:,1) = q0;
pos_n(:,1) = pos_n0;
pos_prev = pos_n0; 
vel_prev = vel_n0; 
att_n(:,1) = quat2eul(q0'); 
velb(:,1) = vel_b0; 

pos_est(:,1) = pos_n0;
vel_est(:,1) = vel_n0; 
bias_est(1,1) = 1; 
bias2_est(:,1) = bias_20;

pos_est2(:,1) = pos_est(:,1);  
vel_est2(:,1) = vel_est(:,1); 
bias_est2(1,1) = bias_est(1,1);  
bias2_est2(:,1) = bias_20; 

% Used for results in master's per 04.05.2017 
trans.p1 = [10,-3.25,-0.5]'; 
trans.p2 = [-10,-1.75,-0.8]'; 
trans.p3 = [10, 3.35, -0.3]'; 
trans.p4 = [-10,1.75,0]'; 


%% Generate input struct
% Define class objects
sim = simulator; 
r   = range; 
est = lkf;
est2 = ekf; 
ltv = ltvk; 
mes = measure; 
mes.l = [0;0;(1.5+qualisys(4,2))];

ltv.h = h; 
ltv.acu = 50; 
est.h = h; 
est2.h = h; 
est2.acu = 50; 

mes.a_prev = zeros(3,1); 
mes.y_prev = ones(4,1); 
est.acu = 50;  
alpha = 0.01; 

est.x_bar  = [pos_est(:,1) ; bias_est(1,1) ; vel_est(:,1) ; bias2_est(:,1)] ;
ltv.x_bar  = [pos_est(:,1) ; bias_est(1,1) ; vel_est(:,1) ; bias2_est(:,1)] ;
est2.x_bar = [pos_est2(:,1); bias_est2(1,1); vel_est2(:,1); bias2_est2(:,1)];

%% Run simulations
for i = 1:N-1
    
    % Generate measurement vector 
    mes.get_measurements(qualisys, imu, i, trans);
    pos_n(:,i+1) = mes.pos; 
    
    %vel_n(:,i) = (mes.pos-pos_prev)/h; 
    if ~isnan(vel_prev)
        vel_n(:,i+1) = alpha*((mes.pos-pos_prev)/h)+(1-alpha)*vel_prev; 
        vel_prev = vel_n(:,i+1); 
    else 
        vel_n(:,i+1) = (mes.pos-pos_prev)/h; 
        vel_prev = vel_n(:,i+1); 
    end
    
    pos_prev = mes.pos; 
    acc_n(:,i+1) = mes.R*mes.a; 
    
    % Sigma 1: Generate range measurements/perform algebraic transformation
    r.update2(trans,mes) 
    
    % Sigma 2: LTV Kalman filter 
    ltv.update(r,mes); 
    
    % Sigma 4: Estimate using linearized Kalman filter  
    est.update(trans,ltv,mes); 
    pos_est(:,i+1) = est.x_est(1:3);
    bias_est(1,i+1) = est.x_est(4); 
    vel_est(:,i+1) = est.x_est(5:7); 
    bias2_est(:,i+1) = est.x_est(8:10); 
    %stage3(:,i+1) = est.x_est(1:3);
    
    % Compare with EKF 
    est2.update(trans,mes); 
    pos_est2(:,i+1) = est2.x_est(1:3);
    bias_est2(1,i+1) = est2.x_est(4); 
    vel_est2(:,i+1) = est2.x_est(5:7); 
    bias2_est2(:,i+1) = est2.x_est(8:10); 
    
 
end

%% Plot of graphs

figure()
subplot(311), plot(imu(1,:), acc_n(1,:)), xlabel('Time [m]'), ylabel('Acceleration in North [m/s^2]'); 
subplot(312), plot(imu(1,:), acc_n(1,:)), xlabel('Time [m]'), ylabel('Acceleration in East [m/s^2]'); 
subplot(313), plot(imu(1,:), acc_n(3,:)), xlabel('Time [m]'), ylabel('Acceleration in Down [m/s^2]');  

figure()
plot(pos_est(2,:), pos_est(1,:), 'g', pos_est2(2,:), pos_est2(1,:),'b', qualisys(3,:), qualisys(2,:), '-r');
legend('TSF', 'EKF', 'Qualisys'), title('Trajectory in 2D'), xlabel('East [m]'), ylabel('North [m]'); 

figure()
subplot(211), plot(pos_est(2,:), pos_est(1,:), pos_n(2,:), pos_n(1,:), '+'), legend('TSF', 'Qualisys measurements-shifted'), title('Trajectory in 2D'), xlabel('East [m]'),ylabel('North [m]'); 
subplot(212), plot(imu(1,:), pos_est(3,:), 'm', imu(1,:), -1.5+imu(8,:), '-g', imu(1,:), pos_n(3,:), 'bo'), legend('TSF', 'Depth measurement-shifted', 'Qualisys measurement-shifted'), xlabel('Time [s]'), ylabel('Down [m]'); 

figure()
subplot(211), plot(pos_est2(2,:), pos_est2(1,:), pos_n(2,:), pos_n(1,:), '+'), legend('EKF', 'Qualisys measurements'), title('Trajectory in 2D'), xlabel('East [m]'),ylabel('North [m]'); 
subplot(212), plot(imu(1,:), pos_est2(3,:), 'm', imu(1,:), -1.5+imu(8,:), '-g', imu(1,:), pos_n(3,:), 'bo'), legend('EKF', 'Depth measurement-shifted', 'Qualisys measurement-shifted'), xlabel('Time [s]'), ylabel('Down [m]'); 

figure()
%plot3(qualisys(2,:), -qualisys(3,:), -imu(8,:)); 
plot3(pos_n(1,:), pos_n(2,:), pos_n(3,:),'o');
xlabel('North [m]'),ylabel('East [m]'),zlabel('Up [m]'); 
hold on;
plot3(pos_est(1,:), pos_est(2,:), pos_est(3,:)); 
plot3(trans.p1(1), trans.p1(2), trans.p1(3), 'ko'); 
plot3(trans.p2(1), trans.p2(2), trans.p2(3), 'ko'); 
plot3(trans.p3(1), trans.p3(2), trans.p3(3), 'ko'); 
plot3(trans.p4(1), trans.p4(2), trans.p4(3), 'ko'); 
title('Plot of trajectory in 3D')

figure()
subplot(311), plot(qualisys(1,:), pos_est(1,:), 'b-.', qualisys(1,:), pos_n(1,:), 'r'), legend('TSF', 'Qualisys'), xlabel('Time [s]'), ylabel('North [m]');
subplot(312), plot(qualisys(1,:), pos_est(2,:), 'b-.', qualisys(1,:), pos_n(2,:), 'r'), legend('TSF', 'Qualisys'), xlabel('Time [s]'), ylabel('East [m]');
subplot(313), plot(qualisys(1,:), pos_est(3,:), 'b-.', qualisys(1,:), pos_n(3,:), 'r'), legend('TSF', 'Qualisys'), xlabel('Time [s]'), ylabel('Down [m]');

figure()
subplot(311), plot(qualisys(1,:), pos_est2(1,:), 'b-.', qualisys(1,:), pos_n(1,:), 'r'), legend('EKF', 'Qualisys'), xlabel('Time [s]'), ylabel('North [m]');
subplot(312), plot(qualisys(1,:), pos_est2(2,:), 'b-.', qualisys(1,:), pos_n(2,:), 'r'), legend('EKF', 'Qualisys'), xlabel('Time [s]'), ylabel('East [m]');
subplot(313), plot(qualisys(1,:), pos_est2(3,:), 'b-.', qualisys(1,:), pos_n(3,:), 'r'), legend('EKF', 'Qualisys'), xlabel('Time [s]'), ylabel('Down [m]');

figure()
subplot(311), plot(qualisys(1,:), pos_est2(1,:), 'b-.', qualisys(1,:), pos_n(1,:), 'r'), legend('EKF', 'Qualisys - shifted'), xlabel('Time [s]'), ylabel('North [m]');
subplot(312), plot(qualisys(1,:), pos_est2(2,:), 'b-.', qualisys(1,:), pos_n(2,:), 'r'), legend('EKF', 'Qualisys - shifted'), xlabel('Time [s]'), ylabel('East [m]');
subplot(313), plot(qualisys(1,:), pos_est2(3,:), 'b-.', qualisys(1,:), pos_n(3,:), 'r'), legend('EKF', 'Qualisys - shifted'), xlabel('Time [s]'), ylabel('Down [m]');


figure()
subplot(311), plot(qualisys(1,:), pos_est(1,:), qualisys(1,:), pos_est2(1,:)), legend('TSF','EKF'), xlabel('Time [s]'), ylabel('North [m]');
subplot(312), plot(qualisys(1,:), pos_est(2,:), qualisys(1,:), pos_est2(2,:)), legend('TSF','EKF'), xlabel('Time [s]'), ylabel('East [m]');
subplot(313), plot(qualisys(1,:), pos_est(3,:), qualisys(1,:), pos_est2(3,:)), legend('TSF','EKF'), xlabel('Time [s]'), ylabel('Down [m]');

figure()
plot(qualisys(1,:), bias_est, qualisys(1,:), bias_est2), legend('TSF', 'EKF'), title('Wave speed estimate'), xlabel('Time [s]'), ylabel('Beta [-]'); 

figure()
subplot(311), plot(imu(1,:), bias2_est(1,:), imu(1,:), bias2_est2(1,:)), legend('TSF', 'EKF'), xlabel('Time [s]'), ylabel('Bias udot [m/s^2]');
subplot(312), plot(imu(1,:), bias2_est(2,:), imu(1,:), bias2_est2(2,:)), legend('TSF', 'EKF'), xlabel('Time [s]'), ylabel('Bias vdot [m/s^2]');
subplot(313), plot(imu(1,:), bias2_est(3,:), imu(1,:), bias2_est2(3,:)), legend('TSF', 'EKF'), xlabel('Time [s]'), ylabel('Bias wdot [m/s^2]');

errorx =  pos_n(1,:) - pos_est(1,:);
errory =  pos_n(2,:) - pos_est(2,:); 
errorz =  pos_n(3,:) - pos_est(3,:); 

errorx2 =  pos_n(1,:) - pos_est2(1,:);
errory2 =  pos_n(2,:) - pos_est2(2,:); 
errorz2 =  pos_n(3,:) - pos_est2(3,:); 

figure()
subplot(311), plot(qualisys(1,:), errorx, '-g', qualisys(1,:), errorx2, 'b'), xlabel('Time [s]'), ylabel('Error in North [m]'), title('North position error'), legend('TSF', 'EKF');
subplot(312), plot(qualisys(1,:), errory, '-g', qualisys(1,:), errory2, 'b'), xlabel('Time [s]'), ylabel('Error in East [m]'), title('East position error'), legend('TSF', 'EKF');
subplot(313), plot(qualisys(1,:), errorz, '-g', qualisys(1,:), errorz2, 'b'), xlabel('Time [s]'), ylabel('Error in Down [m]'), title('Down position error'), legend('TSF', 'EKF');

figure()
subplot(311), plot(qualisys(1,:), vel_est(1,:), '-g', qualisys(1,:), vel_est2(1,:), 'b'), xlabel('Time [s]'), ylabel('North Velocity [m/s]'), title('North Velocity'), legend('TSF', 'EKF');
subplot(312), plot(qualisys(1,:), vel_est(2,:), '-g', qualisys(1,:), vel_est2(2,:), 'b'), xlabel('Time [s]'), ylabel('East Velocity [m/s]'), title('East Velocity'), legend('TSF', 'EKF');
subplot(313), plot(qualisys(1,:), vel_est(3,:), '-g', qualisys(1,:), vel_est2(3,:), 'b'), xlabel('Time [s]'), ylabel('Down Velocity [m/s]'), title('Down Velocity'), legend('TSF', 'EKF');

figure()
subplot(311), plot(qualisys(1,:), vel_est(1,:), 'g', qualisys(1,:), vel_est2(1,:), 'b', qualisys(1,:), vel_n(1,:),'r'), title('North velocity'), legend('TSF', 'EKF', 'QC derivative'), xlabel('Time [s]'), ylabel('North Velocity [m/s]');
subplot(312), plot(qualisys(1,:), vel_est(2,:), 'g', qualisys(1,:), vel_est2(2,:), 'b', qualisys(1,:), vel_n(2,:),'r'), title('East velocity'), legend('TSF', 'EKF', 'QC derivative'), xlabel('Time [s]'), ylabel('East Velocity [m/s]');
subplot(313), plot(qualisys(1,:), vel_est(3,:), 'g', qualisys(1,:), vel_est2(3,:), 'b', qualisys(1,:), vel_n(3,:),'r'), title('Down velocity'), legend('TSF', 'EKF', 'QC derivative'), xlabel('Time [s]'), ylabel('Down Velocity [m/s]');

true = 1450*ones(1,length(bias_est)); 
figure()
plot(qualisys(1,:), true, qualisys(1,:), sqrt(bias_est)*1450, qualisys(1,:), sqrt(bias_est2)*1450), xlabel('Time [s]'), ylabel('Beta [-]'), title('Wave speed estimate'), legend('True','TSF', 'EKF'); 


errorvx =  vel_n(1,:) - vel_est(1,:);
errorvy =  vel_n(2,:) - vel_est(2,:); 
errorvz =  vel_n(3,:) - vel_est(3,:); 

errorvx2 =  vel_n(1,:) - vel_est2(1,:);
errorvy2 =  vel_n(2,:) - vel_est2(2,:); 
errorvz2 =  vel_n(3,:) - vel_est2(3,:); 

figure()
subplot(311), plot(qualisys(1,:), errorvx, '-g', qualisys(1,:), errorvx2, 'b'), xlabel('Time [s]'), ylabel('Error in North velocity [m/s]'), title('North velocity error'), legend('TSF', 'EKF');
subplot(312), plot(qualisys(1,:), errorvy, '-g', qualisys(1,:), errorvy2, 'b'), xlabel('Time [s]'), ylabel('Error in East velocity [m/s]'), title('East velocity error'), legend('TSF', 'EKF');
subplot(313), plot(qualisys(1,:), errorvz, '-g', qualisys(1,:), errorvz2, 'b'), xlabel('Time [s]'), ylabel('Error in Down velocity [m/s]'), title('Down velocity error'), legend('TSF', 'EKF');

