%% Initialise parameters
% Classes: 
%
% simulator: updates position and velocity in NED
% range: generates range measurements (algebraic transformation) 
% ekf: Extended Kalman Filter estimating position, velocity and bias
%
% Date for latest update: 06.02.2017
% Programmed by: Katrine Seel
% Master thesis, Spring 2017 

%% Load measurement data 

clear all; 
read_data; 

t_end = imu(end,1); 
h = 0.009; 
M = size(imu,1);
K = size(qualisys,1);
J = size(range,1); 

% Define initial conditions
vel_b0 = [0 0 0]'; 
acc_n0 = [0 0 0]';
vel_n0 = [0 0 0]';  
q0     = [1;0;0;0];         % equals 0 roll, pitch, and yaw 
pos_n0 = qualisys(1,2:4)'; 
bias_20 = [0;0;0]; 
pos_n(:,1) = pos_n0; 
time(1,1) = 0; 
vel_prev = vel_n0; 
pos_prev = pos_n0; 
acc_prev = [0,0,-9.81]; 

q(:,1) = q0;
pos_n(:,1) = pos_n0;
att_n(:,1) = quat2eul(q0'); 
velb(:,1) = vel_b0; 

pos_est(:,1) = [-30;-20;15];
vel_est(:,1) = [0.1;-0.1063;0.2]; 
bias_est(1,1) = 0.8751; 
bias2_est(:,1) = bias_20;

pos_est2(:,1) = pos_est(:,1);  
vel_est2(:,1) = vel_est(:,1); 
bias_est2(1,1) = bias_est(1,1);  
bias2_est2(:,1) = bias_20; 

error1(2:4,1) = pos_n0-pos_est(:,1); 
error2(2:4,1) = pos_n0-pos_est2(:,1);

% Define transponder positions
trans.p1 = [-2.356,1.259,-0.757]';    
trans.p2 = [2.965,1.009,-0.148]'; 
trans.p3 = [2.150,-0.677,-0.769]'; 
trans.p4 = [-1.642,-1.610,-0.193]';  

%% Generate input struct
% Define class objects
sim = simulator; 
r = algtrans; 
est = lkf;
est2 = ekf; 
ltv = ltvk; 
%est = lkf; 
mes = measure; 

ltv.h = h; 
ltv.acu = 50; 
est.h = h; 
est2.h = h; 
est2.acu = 100; 
 
m = 1; % counter for imu measurements
k = 1; % counter for qualisys measurements
j = 1; % counter for range measurements 
i = 1; % counter for estimated data 

range_mes = 0; 
input.range = 0; 
input.rot = eye(3); 
input.imu = 0; 
input.pos = 0; 

freq.imu = 100;
freq.range = 0.8; 
freq.observer = 150; 

est.x_bar  = [pos_est(:,1) ; bias_est(1,1) ; vel_est(:,1) ; bias2_est(:,1)] ;
ltv.x_bar  = [pos_est(:,1) ; bias_est(1,1) ; vel_est(:,1) ; bias2_est(:,1)] ;
est2.x_bar = [pos_est2(:,1); bias_est2(1,1); vel_est2(:,1); bias2_est2(:,1)];

alpha = 0.01; 

timer.time = imu(1,1); 
timer.imu = tic; 
timer.range = tic; 
timer.qc = tic; 
timer.observer = tic; 

%% Run simulations
while timer.time < t_end && m~=M && k~=K && j~=J 
    
    timer.loop = tic;  
    get_data; 
    
    if input.imu 
        
        %Sigma 1: Generate range measurements/perform algebraic transformation
        if input.range
            r.update2(trans,input)
        end
        
        % Sigma 2: LTV Kalman filter 
        ltv.update(r,input); 
        
        % Sgima 4: Estimate using linearized Kalman filter  
        est.update(trans,ltv,input); 
        pos_est(:,i+1) = est.x_est(1:3);
        bias_est(1,i+1) = est.x_est(4); 
        vel_est(:,i+1) = est.x_est(5:7); 
        bias2_est(:,i+1) = est.x_est(8:10); 

        % Compare with EKF
        est2.update(trans,input); 
        pos_est2(:,i+1) = est2.x_est(1:3);
        bias_est2(1,i+1) = est2.x_est(4); 
        vel_est2(:,i+1) = est2.x_est(5:7); 
        bias2_est2(:,i+1) = est2.x_est(8:10); 
        
        time(1,i+1) = timer.time; 
        i = i+1;
        timer.observer = tic; 
        
    end
    
    
    if input.pos
        
        % Find position estimation error
        error1(1,k) = qualisys(k,1); 
        error2(1,k) = error1(1,k); 
        error1(2:4,k) = pos_n(:,k)-pos_est(:,i); 
        error2(2:4,k) = pos_n(:,k)-pos_est2(:,i);
        
        % Find velocity estimation error
        dt = error1(1,k)-error1(1,k-1); 
        vel_n(:,k) = alpha*((qualisys(k,2:4)'-pos_prev)/(dt))-(1-alpha)*vel_prev; 
        errorv1(1:3,k) = vel_n(:,k)-vel_est(:,i); 
        errorv2(1:3,k) = vel_n(:,k)-vel_est2(:,i);
        vel_prev = vel_n(:,k); 
        pos_prev = pos_n(:,k); 
        
    end
    
    input.pos = 0; 
    input.imu = 0; 
    timer.time = timer.time + toc(timer.loop); 
       
end

%% Plot of graphs

figure()
plot(pos_est2(1,:), pos_est2(2,:), 'b', pos_n(1,:), pos_n(2,:), 'ro'), legend('EKF', 'Qualisys'), xlabel('North [m]'), ylabel('East [m]');

figure()
subplot(311), plot(time(1,:), pos_est(1,:), time(1,:), pos_est2(1,:)), xlabel('Time [s]'), ylabel('North position'), legend('TSF', 'EKF'); 
subplot(312), plot(time(1,:), pos_est(2,:), time(1,:), pos_est2(2,:)), xlabel('Time [s]'), ylabel('East position'), legend('TSF', 'EKF'); 
subplot(313), plot(time(1,:), pos_est(3,:), time(1,:), pos_est2(3,:)), xlabel('Time [s]'), ylabel('Down position'), legend('TSF', 'EKF'); 


figure()
subplot(311), plot(error1(1,:), error1(2,:), 'g', error2(1,:), error2(2,:), 'b'), xlabel('Time [s]'), ylabel('Error in North [m]'), title('North position error'), legend('TSF', 'EKF'); 
subplot(312), plot(error1(1,:), error1(3,:), 'g', error2(1,:), error2(3,:), 'b'), xlabel('Time [s]'), ylabel('Error in East [m]'), title('East position error'), legend('TSF', 'EKF'); 
subplot(313), plot(error1(1,:), error1(4,:), 'g', error2(1,:), error2(4,:), 'b'), xlabel('Time [s]'), ylabel('Error in Down [m]'), title('Down position error'), legend('TSF','EKF'); 

figure()
subplot(311), plot(time(1,:), bias2_est(1,:), 'm', time(1,:), bias2_est2(1,:), 'c'), legend('TSF','EKF'), xlabel('Time [s]'), ylabel('Bias udot [m/s^2]');
subplot(312), plot(time(1,:), bias2_est(2,:), 'm', time(1,:), bias2_est2(2,:), 'c'), legend('TSF','EKF'), xlabel('Time [s]'), ylabel('Bias vdot [m/s^2]');
subplot(313), plot(time(1,:), bias2_est(3,:), 'm', time(1,:), bias2_est2(3,:),'c'), legend('TSF','EKF'), xlabel('Time [s]'), ylabel('Bias wdot [m/s^2]');

figure()
subplot(211), plot(time(1,:), sqrt(bias_est)*1450, time(1,:), sqrt(bias_est2)*1450), legend('TSF','EKF'), xlabel('Time [s]'), ylabel('Beta [-]'), title('Wave speed estimate');
subplot(212), plot(time(1,:), bias_est, time(1,:), bias_est2), legend('TSF','EKF'), xlabel('Time [s]'), ylabel('Beta [-]'), title('Beta');


figure()
subplot(311), plot(imu(:,2)); 
subplot(312), plot(imu(:,3)); 
subplot(313), plot(imu(:,4)); 



figure()
%subplot(311), plot(time(1,:), vel_est(1,:), time(1,:), vel_est2(1,:)), xlabel('Time [s]'), ylabel('North position'), legend('TSF', 'EKF'); 
subplot(311), plot(time(1,:), vel_est(1,:), time(1,:), vel_est2(1,:)), xlabel('Time [s]'), ylabel('North position'), legend('TSF'); 
subplot(312), plot(time(1,:), vel_est(2,:), time(1,:), vel_est2(2,:)), xlabel('Time [s]'), ylabel('East position'), legend('TSF', 'EKF'); 
subplot(313), plot(time(1,:), vel_est(3,:), time(1,:), vel_est2(3,:)), xlabel('Time [s]'), ylabel('Down position'), legend('TSF', 'EKF'); 

figure()
subplot(311), plot(error1(1,:), pos_n(1,:), time(1,:), pos_est(1,:)); 
subplot(312), plot(error1(1,:), pos_n(2,:), time(1,:), pos_est(2,:)); 
subplot(313), plot(error1(1,:), pos_n(3,:), time(1,:), pos_est(3,:)); 

figure()
subplot(311), plot(error1(1,:), vel_n(1,:), 'r'), ylabel('True vel North [m/s]','fontsize', 14), xlabel('Time [s]', 'fontsize', 14);
subplot(312), plot(error1(1,:), vel_n(2,:), 'r'), ylabel('True vel East [m/s]','fontsize', 14), xlabel('Time [s]', 'fontsize', 14);
subplot(313), plot(error1(1,:), vel_n(3,:), 'r'), ylabel('True vel Down [m/s]','fontsize', 14), xlabel('Time [s]', 'fontsize', 14);


figure()
subplot(311), plot(error1(1,:), errorv1(1,:), 'g', error1(1,:), errorv2(1,:), 'b' ), ylabel('Velocity error North [m/s]','fontsize', 14), xlabel('Time [s]', 'fontsize', 14);
subplot(312), plot(error1(1,:), errorv1(2,:), 'g', error1(1,:), errorv2(2,:), 'b' ), ylabel('Velocity error East [m/s]','fontsize', 14), xlabel('Time [s]', 'fontsize', 14);
subplot(313), plot(error1(1,:), errorv1(3,:), 'g', error1(1,:), errorv2(3,:), 'b' ), ylabel('Velocity error Down [m/s]','fontsize', 14), xlabel('Time [s]', 'fontsize', 14);


