%% Initialise parameters
% Classes: 
%
% simulator: create motion of ROV and generate necessary measurements
% range: generates range measurements (algebraic transformation) 
% lkf/ktv : Three-Stage Filter estimating position, velocity and bias
% ekf: Extended Kalman Filter estimating position, velocity and bias
%
% Date for latest update: 05.06.2017
% Programmed by: Katrine Seel
% Master thesis, Spring 2017 

clear all; 

t_end = 300;             % simulation run length seconds
h = 0.01;               % time step in seconds
N = floor(t_end/h);   

% Allocation of matrices
q = zeros(4,N);         
pos_n = zeros(3,N);
vel_n = zeros(3,N);
acc_n = zeros(3,N);
vel_b  = zeros(3,N); 
att_n = zeros(3,N); 
pos_est = zeros(3,N);
vel_est = zeros(3,N); 
bias_est = zeros(1,N); 
pos_gen = zeros(4,N);
p = zeros(6,N); 
pos_est2 = zeros(3,N);
vel_est2 = zeros(3,N); 
bias_est2 = zeros(1,N);
bias2_est = zeros(3,N); 
bias2_est2 = zeros(3,N);
cond = zeros(1,N); 

% Define initial conditions
vel_b0 = [0 0 0]'; 
acc_n0 = [0 0 0]';
vel_n0 = [0 0 0]';  
q0     = [1;0;0;0];   
pos_n0 = [0;0;0]; 
bias_20 = [0.1; 0.05; -0.13]; 

q(:,1) = q0;
pos_n(:,1) = pos_n0;
att_n(:,1) = quat2eul(q0'); 
velb(:,1) = vel_b0; 

% Transient behaviour 
pos_est(:,1) = [10;-15;-10]; 
vel_est(:,1) = [0.1;0.2;0.1];
bias_est(1,1) = 0.8751;
bias2_est(:,1) = [0;0;0]; 

pos_est2(:,1) = pos_est(:,1);  
vel_est2(:,1) = vel_est(:,1); 
bias_est2(1,1) = bias_est(1,1);   
bias2_est2(:,1) = bias2_est(:,1); 

% Plotting the different stages of the filter 
stage1 = zeros(4,N);
stage1_2 = zeros(4,N); 
stage2 = zeros(3,N);
stage3 = zeros(3,N); 
stage1(:,1) = [pos_est(:,1); bias_est(1,1)];
stage1_2(:,1) = [pos_est(:,1); bias_est(1,1)];
stage2(:,1) = pos_est(:,1); 
stage3(:,1) = pos_est(:,1);


% More realistic transponder positions (placed on the sea-bed)
trans.p1 = [10,10,0]';   
trans.p2 = [10,-10,1]'; 
trans.p3 = [-10,10,2]'; 
trans.p4 = [-10,-10,0]'; 

% Initialize trajectory 
% Defined in body (acceleration and angular velocities) because easier
dvel_b = zeros(3,t_end/h);  % Desired acceleration in BODY
omega  = zeros(3,t_end/h);  % Desired angular velocity in BODY 
velv_b  = zeros(3,t_end/h);  % Desired velocity in BODY 
l = 6000;
t = 500;
s = 6000;
%b = 40000;
b = 0; 
%dw = pi/100; % used for circular path 
dw = pi/10; 

 
%% Generate trajectory--------------------------------------------%
% tra = 1 : Straight-line trajectory  
% tra = 2 : Lawn mower trajectory (does not move the first 400 s)
% tra = 3 : Lawn mower trajectory (starts moving instantaneously)
% tra = 4 : Square trajectory 
%-----------------------------------------------------------------%

% Parameter selecting trajectory 
tra = 3;
[dvel_b, omega,velv_b] = generate_trajectory(dvel_b, omega,velv_b, l, t, s, b, dw, tra); 

%% Generate input struct
% Define class objects
sim = simulator; 
r   = range; 
est = lkf;
est2 = ekf; 
ltv = ltvk; 

r.z_prev = pos_est(3,1); 

ltv.h = h; 
ltv.acu = 50;

est.h = h; 
est.acu = 50; % adjust frequency for range measurements 

est2.h = h; 
est2.acu = 50; % adjust frequency for range measurements 
 
sim.posn = pos_n0;
sim.q = q0; 
sim.h = h;
sim.velb = vel_b0; 
sim.acc_n = acc_n0; 
sim.acc_b = acc_n0; 
sim.veln = vel_n0;
sim.b = bias_20; 
sim.Rnb = eye(3); 

input.dvel = dvel_b(:,1); 
input.w = omega(:,1); 
input.vel = velv_b(:,1); 

ltv.x_bar  = [pos_est(:,1)  ; bias_est(1,1)  ; vel_est(:,1) ; bias2_est(:,1)] ;
est.x_bar  = [pos_est(:,1)  ; bias_est(1,1)  ; vel_est(:,1) ; bias2_est(:,1)] ;
est2.x_bar = [pos_est2(:,1) ; bias_est2(1,1) ; vel_est2(:,1); bias2_est2(:,1)];


%% Run simulations
for i = 1:N-1
 
    % Sigma 1: Generate range measurements/perform algebraic transformation 
    r.update2(sim,trans); 
    stage1(:,i+1) = r.xm;  
    stage1_2(:,i+1) = r.x2; 
   
    % Sigma 2: LTV Kalman filter 
    ltv.update(r,sim); 
    stage2(:,i) = ltv.x_est(1:3);  
    
    % Sgima 4: Estimate using linearized Kalman filter
    est.update(r,sim,trans,ltv); 
    pos_est(:,i+1) = est.x_est(1:3);
    bias_est(1,i+1) = est.x_est(4); 
    vel_est(:,i+1) = est.x_est(5:7); 
    bias2_est(:,i+1) = est.x_est(8:10); 
    stage3(:,i+1) = est.x_est(1:3);  
    
    % Compare with EKF 
    est2.update(r,sim,trans); 
    pos_est2(:,i+1) = est2.x_est(1:3);
    bias_est2(1,i+1) = est2.x_est(4); 
    vel_est2(:,i+1) = est2.x_est(5:7); 
    bias2_est2(:,i+1) = est2.x_est(8:10); 
    
    % Update real motion of the vehicle 
    sim.update(input); 
    
    % Store position, velocity and attitude values
    pos_n(:,i+1) = sim.posn; 
    vel_b(:,i+1) = sim.velb; 
    vel_n(:,i+1) = sim.veln; 
    att_n(:,i+1) = quat2eul(sim.q'); 
   
    % Update linear and angular velocity 
    input.dvel = dvel_b(:,i+1); 
    input.w = omega(:,i+1); 
    input.vel = velv_b(:,i+1); 
    
end

%% Plot of graphs

generate_plots(t_end, h, trans, pos_n,vel_n, pos_est, vel_est, bias_est, pos_est2,vel_est2, bias_est2, stage1, stage1_2, stage2, stage3, bias2_est, bias_20, bias2_est2); 

